/* ****************************************************************************
* Copyright (c) TCC - 2009 IME - USP. 
*
* This source code is subject to terms and conditions of the Microsoft Public License. 
* A copy of the license can be found in http://msdn.microsoft.com/en-us/library/dd228915.aspx. 
* By using this source code in any fashion, you are agreeing to be bound by the terms of the 
* Microsoft Public License.
*
* You must not remove this notice, or any other, from this software.
* ***************************************************************************/

using System;
using System.Diagnostics;
using System.IO;


namespace SoundLibrary
{
    public class Mp3Stream : Stream
    {
        private Bitstream JZBitStream;
        private Stream SourceStream;
        private OBuffer16BitStereo QueueOBuffer;
        protected SoundFormat FormatRep = SoundFormat.Pcm16BitStereo;
        private Decoder JZDecoder = new Decoder(Decoder.DefaultParams);
        private int FrequencyRep = -1;
        private short ChannelCountRep = -1;

        #region Properties

        public int Frequency
        {
            get { return FrequencyRep; }
        }
        
        public short ChannelCount
        {
            get { return ChannelCountRep; }
        }

        public SoundFormat Format
        {
            get { return FormatRep; }
        }

        #endregion

        #region Constructors

        public Mp3Stream(string fileName): this(new FileStream(fileName, FileMode.Open))
        {
        }

        public Mp3Stream(string fileName, int chunkSize): this(new FileStream(fileName, FileMode.Open), chunkSize)
        {
        }

        public Mp3Stream(Stream sourceStream): this(sourceStream, 4096)
        {
        }

        public Mp3Stream(Stream sourceStream, int chunkSize)
        {
            FormatRep = SoundFormat.Pcm16BitStereo;
            SourceStream = sourceStream;
            JZBitStream = new Bitstream(new BackStream(SourceStream, chunkSize));
            QueueOBuffer = new OBuffer16BitStereo();
            JZDecoder.OutputBuffer = QueueOBuffer;
        }

        #endregion

        #region Stream Override

        public override bool CanRead
        {
            get { return SourceStream.CanRead; }
        }

        public override bool CanSeek
        {
            get { return SourceStream.CanSeek; }
        }

        public override bool CanWrite
        {
            get { return SourceStream.CanWrite; }
        }

        public override long Length
        {
            get { return SourceStream.Length; }
        }

        public override void Flush()
        {
            SourceStream.Flush();
        }

        public override long Position
        {
            get { return SourceStream.Position; }
            set { SourceStream.Position = value; }
        }

        public override long Seek(long pos, SeekOrigin origin)
        {
            return SourceStream.Seek(pos, origin);
        }

        public override void SetLength(long len)
        {
            throw new InvalidOperationException();
        }

        public override void Write(byte[] buf, int ofs, int count)
        {
            throw new InvalidOperationException();
        }

        /// <summary>
        /// L 'count' bytes a partir de 'offset' e os guarda em 'buffer'. Dentro da classe Mp3Stream
        /// existe um buffer do tipo OBuffer18BitStereo, a cada leitura  feita uma verificacao consultando
        /// se este buffer est vazio. Se estiver cheio uma parte deste buffer  copiada para a varivel buffer.
        /// se estiver vazio,  chamada uma funo ReadFrame que l um frame do mp3, decodifica e escreve no
        /// OBuffer18BitStereo.
        /// </summary>
        /// <param name="buffer">Array de bytes que ira receber os bytes lidos</param>
        /// <param name="offset">Indica onde a leitura comea</param>
        /// <param name="count">Tamanho do array buffer</param>
        /// <returns>Devolve o nmero de bytes lidos.</returns>
        public override int Read(byte[] buffer, int offset, int count)
        {
            int bytesRead = 0;
            while (true)
            {
                if (QueueOBuffer.bytesLeft <= 0)
                {
                    if (!ReadFrame())
                        break;
                }

                bytesRead += QueueOBuffer.Read(buffer, offset + bytesRead, count - bytesRead);

                if (bytesRead >= count)
                    break;
            }
            return bytesRead;
        }

        public override void Close()
        {
            JZBitStream.Close();
        }

        #endregion

        #region Public Methods

        public int DecodeFrames(int frameCount)
        {
            int framesDecoded = 0;
            bool aFrameWasRead = true;
            while (framesDecoded < frameCount && aFrameWasRead)
            {
                aFrameWasRead = ReadFrame();
                if (aFrameWasRead) framesDecoded++;
            }
            return framesDecoded;
        }

        #endregion

        #region Private Methods

        private bool ReadFrame()
        {
            Header header = JZBitStream.ReadFrame();
            if (header == null)
            {
                return false;
            }

            try
            {
                if (header.Mode() == Header.SINGLE_CHANNEL)
                    ChannelCountRep = (short)1;
                else
                    ChannelCountRep = (short)2;

                FrequencyRep = header.Frequency();
                Obuffer decoderOutput = JZDecoder.DecodeFrame(header, JZBitStream);
                if (decoderOutput != QueueOBuffer)
                    throw new System.ApplicationException("Output buffers are different.");
            }
            finally
            {
                JZBitStream.CloseFrame();
            }
            return true;
        }

        #endregion

    }

    public enum SoundFormat
    {
        Pcm16BitMono,
        Pcm16BitStereo,
    }

    internal class OBuffer16BitStereo : Obuffer
    {
        static readonly int CHANNELS = 2;
        int _offset;
        int _end;
        byte[] buffer = new byte[OBUFFERSIZE * 2];
        int[] bufferp = new int[MAXCHANNELS];

        #region Contructor

        public OBuffer16BitStereo()
        {
            clear_buffer();
        }

        #endregion

        #region Public Methods

        public int bytesLeft
        {
            get { return _end - _offset; }
        }

        public int Read(byte[] buffer_out, int offset, int count)
        {
            int remaining = bytesLeft;
            int copySize;
            if (count > remaining)
            {
                copySize = remaining;
                Debug.Assert(copySize % (2 * CHANNELS) == 0);
            }
            else
            {
                int remainder = count % (2 * CHANNELS);
                copySize = count - remainder;
            }

            Array.Copy(buffer, _offset, buffer_out, offset, copySize);

            _offset += copySize;
            return copySize;
        }

        #endregion

        #region Obuffer Override

        public override void append(int channel, short value)
        {
            buffer[bufferp[channel]] = (byte)(value & 0xff);
            buffer[bufferp[channel] + 1] = (byte)(value >> 8);
            bufferp[channel] += CHANNELS * 2;
        }

        public override void appendSamples(int channel, float[] f)
        {
            int pos = bufferp[channel];

            float fs;
            for (int i = 0; i < 32; i++)
            {
                fs = f[i];
                if (fs > 32767.0f)
                    fs = 32767.0f;
                else if (fs < -32767.0f)
                    fs = -32767.0f;

                int sample = (int)fs;
                buffer[pos] = (byte)(sample & 0xff);
                buffer[pos + 1] = (byte)(sample >> 8);

                pos += CHANNELS * 2;
            }

            bufferp[channel] = pos;
        }

        public override void clear_buffer()
        {
            _offset = 0;
            _end = 0;

            for (int i = 0; i < CHANNELS; i++)
                bufferp[i] = i * 2;
        }

        public override void set_stop_flag()
        {
        }

        public override void write_buffer(int val)
        {
            _offset = 0;
            _end = bufferp[0];
        }

        public override void close()
        {
        }

        #endregion
    }
}
